{% extends "base.html" %}
{% block html_lang %}{{ article.lang }}{% endblock %}
{% block head -%}
  {{ super() -}}
  {% if article.summary %}
    <meta name="description" content="{{ article.summary | striptags | safe | truncate(150) }}" />
  {% endif %}
{% endblock %}

{% block title %}{{ article.title|striptags }}{% endblock %}

{% block extra_head %}
  {% import 'translations.html' as translations with context %}
  {% if translations.entry_hreflang(article) %}
    {{ translations.entry_hreflang(article) }}
  {% endif %}
{% endblock %}

{% block content %}
  <section id="content" class="body">
    <article>
      <header>
        <h1 class="entry-title">
          <a href="{{ SITEURL }}/{{ article.url }}" rel="bookmark"
             title="Permalink to {{ article.title|striptags }}">{{ article.title }}</a></h1>
        {% include 'twitter.html' %}
      </header>

      <div class="entry-content">
        {% include 'article_infos.html' %}
        {{ article.content }}
      </div><!-- /.entry-content -->
      {% if DISQUS_SITENAME and SITEURL and article.status != "draft" %}
        <div class="comments">
          <h2>Comments !</h2>
          <div id="disqus_thread"></div>
          <script type="text/javascript">
            var disqus_shortname = '{{ DISQUS_SITENAME }}';
            var disqus_identifier = '{{ article.url }}';
            var disqus_url = '{{ SITEURL }}/{{ article.url }}';
            (function() {
              var dsq = document.createElement('script'); dsq.type = 'text/javascript'; dsq.async = true;
              dsq.src = '//{{ DISQUS_SITENAME }}.disqus.com/embed.js';
              (document.getElementsByTagName('head')[0] || document.getElementsByTagName('body')[0]).appendChild(dsq);
            })();
          </script>
          <noscript>Please enable JavaScript to view the comments.</noscript>
        </div>
      {% endif %}

    </article>
  </section>
{% endblock %}
