{% extends "base.html" %}
{% block content_title %}{% endblock %}
{% block content %}
    {% if articles %}
        {% for article in articles_page.object_list %}

        {# First item #}
            {% if loop.first and not articles_page.has_previous() %}
                <aside id="featured" class="body">
                    <article>
                        <h1 class="entry-title"><a href="{{ SITEURL }}/{{ article.url }}">{{ article.title }}</a></h1>
                        {% include 'article_infos.html' %}{{ article.content }}{% include 'comments.html' %}
                    </article>
                </aside><!-- /#featured -->
                {% if loop.length > 1 %}
                    <section id="content" class="body">
                        <h1>Other articles</h1>
                        <hr />
                        <ol id="posts-list" class="hfeed">
                {% endif %}
        {# other items #}
            {% else %}
                {% if loop.first %}
                    <section id="content" class="body">
                        <ol id="posts-list" class="hfeed" start="{{ articles_paginator.per_page -1 }}">
                {% endif %}
                <li><article class="hentry">
                    <header>
                        <h1><a href="{{ SITEURL }}/{{ article.url }}" rel="bookmark"
                               title="Permalink to {{ article.title|striptags }}">{{ article.title }}</a></h1>
                    </header>

                    <div class="entry-content">
                        {% include 'article_infos.html' %}
                        {{ article.summary }}
                        <a class="readmore" href="{{ SITEURL }}/{{ article.url }}">read more</a>
                        {% include 'comments.html' %}
                    </div><!-- /.entry-content -->
                </article></li>
            {% endif %}
            {% if loop.last %}
                {% if loop.length > 1 or articles_page.has_other_pages() %}
                    </ol><!-- /#posts-list -->
                    {% if articles_page.has_other_pages() %}
                        {% include 'pagination.html' %}
                    {% endif %}
                    </section><!-- /#content -->
                {% endif %}
            {% endif %}
        {% endfor %}
    {% else %}
        <section id="content" class="body">
            {% if pages %}
                <h2>Pages</h2>
                <ul>
                    {% for page in pages %}
                        <li><a href="{{ SITEURL }}/{{ page.url }}">{{ page.title }}</a></li>
                    {% endfor %}
                </ul>
            {% else %}
                <p>This site currently has no content.</p>
            {% endif %}
        </section>
    {% endif %}
{% endblock content %}
