<footer class="post-info">
        <abbr class="published" title="{{ article.date.isoformat() }}">
                Published: {{ article.locale_date }}
        </abbr>
        {% if article.modified %}
                <br />
                <abbr class="modified" title="{{ article.modified.isoformat() }}">
                        Updated: {{ article.locale_modified }}
                </abbr>
        {% endif %}

        {% if article.authors %}
                <address class="vcard author">
                        By {% for author in article.authors %}
                                <a class="url fn" href="{{ SITEURL }}/{{ author.url }}">{{ author }}</a>
                        {% endfor %}
                </address>
        {% endif %}
        <p>In <a href="{{ SITEURL }}/{{ article.category.url }}">{{ article.category }}</a>.</p>
        {% include 'taglist.html' %}
        {% import 'translations.html' as translations with context %}
        {{ translations.translations_for(article) }}
</footer><!-- /.post-info -->
