{% extends "base.html" %}

{% block title %}{{ SITENAME|striptags }} - Archives{% endblock %}

{% block content %}
    <h2>Archives for {{ SITENAME }}</h2>

    <dl>
        {% for article in dates %}
            <dt>{{ article.locale_date }}</dt>
            <dd><a href="{{ SITEURL }}/{{ article.url }}">{{ article.title }}</a></dd>
        {% endfor %}
    </dl>
{% endblock %}
