{% if GITHUB_URL %}
    <a href="{{ GITHUB_URL }}">
        {% if GITHUB_POSITION != "left" %}
            <img style="position: absolute; top: 0; right: 0; border: 0;" src="https://github.blog/wp-content/uploads/2008/12/forkme_right_red_aa0000.png" alt="Fork me on GitHub" />
        {% else %}
            <img style="position: absolute; top: 0; left: 0; border: 0;" src="https://github.blog/wp-content/uploads/2008/12/forkme_left_white_ffffff.png" alt="Fork me on GitHub" />
        {% endif %}
    </a>
{% endif %}
