{% extends "index.html" %}

{% block title %}{{ SITENAME|striptags }} - {{ category }} category{% endblock %}

{% block content_title %}
    <h2>Articles in the {{ category }} category</h2>
{% endblock %}
